#!/usr/bin/env python

import os
import sys

_HERE = os.path.abspath(os.path.dirname(__file__))

sys.path.insert(0, os.path.join(_HERE, 'charmhelpers'))

from charmhelpers.core.host import (
    service_start,
    service_stop,
    service_restart,
)

from charmhelpers.core.hookenv import (
    Hooks,
    close_port,
    open_port,
    config as config_get,
    log as juju_log,
    charm_dir,
)

from charmhelpers.fetch import (
    apt_install
)

DEFAULT_RSYSLOG_PORT = 514
DEFAULT_RSYSLOG_PATH = os.path.join(os.path.sep, 'etc', 'rsyslog.d')
DEFAULT_LOGROTATE_PATH = os.path.join(os.path.sep, 'etc', 'logrotate.d',
                                      'rsyslog')
DEFAULT_CONFIGURATION_PARAMS = (
    'syslog_rotate',
    'messages_rotate',
    'nova_logs'
)

required_debian_pkgs = [
    'rsyslog',
    'python-jinja2',
]

hooks = Hooks()

try:
    from jinja2 import Environment, FileSystemLoader
except ImportError:
    apt_install(['python-jinja2'], options=['-f'])
    from jinja2 import Environment, FileSystemLoader


def get_template_dir():
    return os.path.join(charm_dir(), 'templates')


def get_config_template(name):
    template_env = Environment(loader=FileSystemLoader(get_template_dir()))
    return template_env.get_template(name)


def get_changed_config():
    changed = {}
    for config_key in DEFAULT_CONFIGURATION_PARAMS:
        changed[config_key] = config_get(config_key)
        juju_log("Configuration key:%s set to value: %s" %
                 (config_key, changed[config_key]))
    return changed


@hooks.hook()
def install():
    apt_install(required_debian_pkgs, options=['-f'])


@hooks.hook("upgrade-charm")
def upgrade_charm():
    install()


@hooks.hook("start")
def start():
    service_start("rsyslog")
    open_port(DEFAULT_RSYSLOG_PORT)
    open_port(DEFAULT_RSYSLOG_PORT, protocol="UDP")


@hooks.hook("stop")
def stop():
    service_stop("rsyslog")
    close_port(DEFAULT_RSYSLOG_PORT)
    close_port(DEFAULT_RSYSLOG_PORT, protocol="UDP")


def update_logrotate_config(**params):
    template_name = "rsyslog.conf"
    template = get_config_template(template_name)

    with open(DEFAULT_LOGROTATE_PATH, 'w') as logrotate_config:
        logrotate_config.write(template.render(**params))


def update_rsyslog_config(**params):
    template_name = "60-aggregator.conf"
    template = get_config_template(template_name)

    with open(os.path.join(DEFAULT_RSYSLOG_PATH,
                           template_name), 'w') as rsyslog_config:
        rsyslog_config.write(template.render(**params))


@hooks.hook("config-changed")
def config_changed():
    config = get_changed_config()

    update_logrotate_config(**config)
    update_rsyslog_config(**config)

    juju_log("Configuration changed, restart required")

    #configuration changed, restart rsyslog
    service_restart("rsyslog")

if __name__ == "__main__":
    hooks.execute(sys.argv)
