#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jorge Niedbalski R. <jorge.niedbalski@canonical.com>'

import os

_HERE = os.path.abspath(os.path.dirname(__file__))

try:
    import unittest
    import mock
except ImportError as ex:
    raise ImportError("Please install unittest and mock modules")

import hooks

TO_PATCH = [
    "apt_install",
    "install",
    "service_restart",
    "service_start",
    "open_port",
    "service_stop",
    "close_port",
    "juju_log",
    "charm_dir",
    "config_get"
]


class HooksTestCase(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.patch_all()

        self.juju_log.return_value = True
        self.apt_install.return_value = True
        self.charm_dir.return_value = os.path.join(_HERE, '..')

    def patch(self, method):
        _m = mock.patch.object(hooks, method)
        _mock = _m.start()
        self.addCleanup(_m.stop)
        return _mock

    def patch_all(self):
        for method in TO_PATCH:
            setattr(self, method, self.patch(method))

    def test_install_hook(self):
        """Check if install hooks is correctly executed
        """
        hooks.hooks.execute(['install'])
        self.apt_install.assert_called_with(['rsyslog', 'python-jinja2'],
                                            options=["-f"])

    def test_upgrade_charm(self):
        """Check if charm upgrade hooks is correctly executed
        """
        hooks.hooks.execute(['upgrade-charm'])
        self.install.assert_called()

    def test_start_charm(self):
        """Check if start hook is correctly executed
        """
        hooks.hooks.execute(['start'])
        self.service_start.assert_called_with("rsyslog")

        expected = [mock.call(514),
                    mock.call(514, protocol="UDP")]

        self.assertEquals(self.open_port.call_args_list,
                          expected)

    def test_stop_charm(self):
        """Check if start hooks is correctly executed
        """
        hooks.hooks.execute(['stop'])

        self.service_stop.assert_called_with("rsyslog")

        expected = [mock.call(514),
                    mock.call(514, protocol="UDP")]

        self.assertTrue(self.close_port.call_args_list == expected)

    def _tpl_has_value(self, template, value):
        return hooks.get_config_template(template).render(
            **hooks.get_changed_config()).find(value) > 0

    def test_config_nova_disabled(self):
        """Check if configuration templates are created correctly \
        ( nova_logs: False )
        """
        self.apt_install.return_value = True
        self.charm_dir.return_value = os.path.join(_HERE, '..')

        config = {
            "syslog_rotate": 7,
            "messages_rotate": 4,
            "nova_logs": False
        }

        def config_side(*args, **kwargs):
            return config[args[0]]

        self.config_get.side_effect = config_side

        self.assertFalse(self._tpl_has_value("60-aggregator.conf", "nova"))
        self.assertTrue(self._tpl_has_value("rsyslog.conf", "7"))
        self.assertTrue(self._tpl_has_value("rsyslog.conf", "4"))

    def test_config_nova_enabled(self):
        """Check if configuration templates are created correctly \
        ( nova_logs: True )
        """
        config = {
            "syslog_rotate": 7,
            "messages_rotate": 4,
            "nova_logs": True
        }

        def config_side(*args, **kwargs):
            return config[args[0]]

        self.config_get.side_effect = config_side
        self.assertTrue(self._tpl_has_value("60-aggregator.conf", "nova"))

    def test_config_changed(self):
        """Check if config-changed is executed correctly"""
        _open = mock.mock_open()

        with mock.patch('__builtin__.open', _open, create=True):
            hooks.config_changed()

        expected = [
            mock.call(os.path.join(hooks.get_template_dir(),
                                   '60-aggregator.conf'), 'rb'),
            mock.call(os.path.join(hooks.DEFAULT_RSYSLOG_PATH,
                                   '60-aggregator.conf'), 'w'),
            mock.call(os.path.join(hooks.get_template_dir(),
                                   'rsyslog.conf'), 'rb'),
            mock.call(hooks.DEFAULT_LOGROTATE_PATH, 'w')
        ]

        self.assertEquals(sorted(_open.call_args_list), sorted(expected))
        self.service_restart.assert_called_once_with("rsyslog")
